	title	IBM PC terminal using interrupts

stack	segment	para stack
	db	64 dup(?)
stack	ends

data	segment
qbuff	db	256	dup(?)
front	dw	?
rear	dw	?
min	dw	?
max	dw	?
m0	db	'Select Baud Rate',0
m1	db	'1.  300',0
m2	db	'2.  600',0
m3	db	'3. 1200',0
m4	db	'4. 2400',0
m5	db	'5. 4800',0
m6	db	'6. 9600',0
vecseg	dw	?			;save int vec
vecoff	dw	?
txdata	equ	3f8h
recdat	equ	3f8h
mcr	equ	3fch
ier	equ	3f9h
lsr	equ	3fdh
intvec	equ	0ch
imask	equ	21h
eoi	equ	20h
ocw2	equ	20h
data	ends

	extrn	chrout:near,mess:near
	extrn	clrscn:near,quit:near
code	segment	public
	assume	cs:code,ds:data

main	proc	far
	mov	ax,data		;set ds
	mov	ds,ax
	call	initq		;queue init
	call	gtbaud		;get baud 
	call	isetup		;int setup
	in	al,imask
	and	al,0efh		;enable irq4
	out	imask,al
key:	mov	ah,1		;if key not
	int	16h		; pressed
	je 	recv		; goto recv
	mov	ah,0		;else
	int	16h		;read key
	cmp	ah,1		;if esc key
	je	done		;goto DOS
	call	send		;send character
recv:	call	checkq		;check queue
	je	key		;if empty,rdkey
	mov	bl,07h		;normal video
	and	al,7fh		;strip parity
	call	chrout		;display char
	jmp	key		;repeat again
done:	in	al,imask
	or	al,10h		;disable irq4
	out	imask,al
	push	ds		;save ds
	mov	dx,vecoff	;restore old
	mov	ds,vecseg	; 0ch int vec
	mov	al,intvec
	mov	ah,25h
	int	21h
	pop	ds		;restore ds
	call	quit		;goto DOS
main	endp

;	display menu
menu	proc	near
	call 	clrscn		;clear screen
	mov	dx,0510h	;row 5,col 16
	mov	si,offset m0
	mov	bl,07h		;normal video
	call	mess		;mess 0
	add	dh,2		;row=row+2
	add	dl,4		;col=col+4
	mov	si,offset m1
	call	mess		;mess 1
	inc	dh		;inc row
	mov	si,offset m2
	call	mess		;mess 2
	inc	dh
	mov	si,offset m3
	call	mess		;mess 3
	inc	dh
	mov	si,offset m4
	call	mess		;mess 4
	inc	dh
	mov	si,offset m5
	call	mess		;mess 5
	inc	dh
	mov	si,offset m6
	call	mess		;mess 6
	mov	dh,5		;row=5
	mov	dl,33		;col=33
	mov	ah,2
	int	10h		;set cursor
	ret
menu	endp

;	get baud rate
gtbaud	proc	near
	call	menu		;display menu
gb1:	mov	ah,0		;wait for key
	int	16h
	cmp	al,31h		; between 1
	jb	gb1
	cmp	al,36h		; and 6
	ja	gb1
	and	al,0fh		;baud code
	inc	al		;between 2-7
	mov	cl,5		;move to
	shl	al,cl		; bits 5-7
	mov	cl,3		;no parity
	or	al,cl		; 8 bits
	mov	ah,0
	mov	dx,0
	int	14h		;set LCR
	call	clrscn		;clear screen
	ret
gtbaud	endp

;	initialize queue
initq	proc	near
	mov	ax,offset qbuff
	mov	front,ax	;front->qbuff
	mov	rear,ax		;rear->qbuff
	mov	min,ax		;min->qbuff
	add	ax,255		;max ->
	mov	max,ax		; qbuff+255
	ret
initq	endp

;	check queue
checkq	proc	near
	mov	si,front
	cmp	si,rear		;if front=rear
	je	cq2		;then empty
	cli			;disable int
	inc	si		;inc front
	cmp	si,max		;if front>max
	jbe	cq1		;then
	mov	si,min		; front=min
cq1:	mov	front,si	;get byte
	mov	al,[si]		; @ front
	test	al,0ffh		;clear Z flag
	sti			;enable int
cq2:	ret
checkq	endp

;	store al in queue
qstore	proc	near
	inc	rear		;inc rear
	mov	si,rear		;if rear>max
	cmp	si,max		;then
	jbe	qs1
	mov	si,min		;rear=min
	mov	rear,si
qs1:	cmp	si,front	;if front=rear
	jne	qs3		;then full
	dec	si		; dec rear
	cmp	si,min		; if rear<min
	jae	qs2		; then rear=max
	mov	si,max
	mov	rear,si
qs2:	ret			;else store
qs3:	mov	[si],al		; at rear
	ret
qstore	endp

;	send char in al to 8250
send	proc	near
	push	ax		;save char
	mov	dx,lsr
sd1:	in	al,dx		;wait for
	test	al,20h		; trans buff
	je	sd1		; to be empty
	mov	dx,txdata
	pop	ax		;get char &
	out	dx,al		; send it
	ret
send	endp

;	interrupt setup
isetup	proc	near
	mov	dx,mcr		;modem cr
	mov	al,0bh		;out2 lo
	out	dx,al		;DTR,RTS lo
	mov	dx,ier
	mov	al,1		;enable recv
	out	dx,al		; interrupts
	push	ds		;save ds
	mov	ah,35h		;save 0ch int vec
	mov	al,intvec
	int	21h
	mov	vecseg,es
	mov	vecoff,bx
	mov	dx,offset intser
	push	cs
	pop	ds
	mov	ah,25h		;set int type
	int	21h		; vector
	pop	ds		;restore ds
	ret
isetup	endp

;	interrupt service routine
intser	proc	near
	push	ax		;save regs
	push	si
	push	dx
	push	ds
	mov	ax,data
	mov	ds,ax		;ds=data
	mov	dx,lsr		;if char ready
	in	al,dx
	test	al,01h
	je	ints1
	mov	dx,recdat	;read it
	in	al,dx
	call	qstore		;put in queue
ints1:	pop	ds
	pop	dx
	pop	si
	mov	al,eoi		;8259 EOI
	out	ocw2,al
	pop	ax
	iret
intser	endp

code	ends
	end	main
